"""Common types."""

import warnings
from typing import Callable, Iterable

try:
    from pydantic.v1 import BaseModel as PydanticBaseModel, root_validator
    from pydantic.v1.generics import GenericModel as PydanticGenericModel
except ImportError:  # pragma: no cover
    from pydantic import BaseModel as PydanticBaseModel, root_validator  # type: ignore
    from pydantic.generics import GenericModel as PydanticGenericModel  # type: ignore

from .exceptions import ExtraArgumentWarning


class BaseModel(PydanticBaseModel):
    """Base model with extra arguments warning."""

    class Config:
        allow_population_by_field_name = True

    @root_validator(pre=True)
    @classmethod
    def _warn_extra(cls, values: dict) -> dict:
        known_fields = set(cls.__fields__.keys())
        known_fields.update({f.alias for f in cls.__fields__.values()})
        for field in values.keys() - known_fields:
            msg = "Unexpected extra argument {} for model {}.{}".format(
                field, cls.__module__, cls.__name__
            )
            warnings.warn(msg, ExtraArgumentWarning, stacklevel=2)
        return values


class GenericModel(BaseModel, PydanticGenericModel):
    """Base generic model with extra arguments warning."""


class BaseId(str):
    """Identifier base class."""

    @classmethod
    def __get_validators__(cls) -> Iterable[Callable]:
        yield cls._validate

    @classmethod
    def _validate(cls, value: str) -> "BaseId":
        return cls(value)

    def __repr__(self) -> str:
        return "{}({})".format(self.__class__.__name__, super().__repr__())
