"""DNSKEY enums."""

from enum import IntEnum, IntFlag, unique
from typing import Set, Tuple, cast

from .translations import gettext_lazy as _


@unique
class DnskeyFlag(IntFlag):
    """Enum of DNSKEY flags.

    See https://www.iana.org/assignments/dnskey-flags/dnskey-flags.xhtml
    """

    bit: int
    label: str

    def __new__(cls, bit: int, label: str = "") -> "DnskeyFlag":
        """Construct item with label."""
        value = 1 << (15 - bit)
        obj = int.__new__(cls, value)
        obj._value_ = value

        obj.bit = bit
        obj.label = label
        return obj

    _UNASSIGNED_0 = (0, _("Unassigned"))
    _UNASSIGNED_1 = (1, _("Unassigned"))
    _UNASSIGNED_2 = (2, _("Unassigned"))
    _UNASSIGNED_3 = (3, _("Unassigned"))
    _UNASSIGNED_4 = (4, _("Unassigned"))
    _UNASSIGNED_5 = (5, _("Unassigned"))
    _UNASSIGNED_6 = (6, _("Unassigned"))
    ZONE = (7, _("ZONE"))
    REVOKE = (8, _("REVOKE"))
    _UNASSIGNED_9 = (9, _("Unassigned"))
    _UNASSIGNED_10 = (10, _("Unassigned"))
    _UNASSIGNED_11 = (11, _("Unassigned"))
    _UNASSIGNED_12 = (12, _("Unassigned"))
    _UNASSIGNED_13 = (13, _("Unassigned"))
    _UNASSIGNED_14 = (14, _("Unassigned"))
    SECURE_ENTRY_POINT = (15, _("Secure Entry Point (SEP)"))

    @property
    def flags(self) -> Set["DnskeyFlag"]:
        """Return individual flags."""
        return {f for f in cast(Tuple["DnskeyFlag", ...], tuple(self.__class__)) if f in self}


@unique
class DnskeyAlgorithm(IntEnum):
    """Enum of DNSKEY algorithms.

    See https://www.iana.org/assignments/dns-sec-alg-numbers/dns-sec-alg-numbers.xhtml
    Member names are based on Mnemonic names of the records.
    """

    label: str

    def __new__(cls, value: int, label: str = "") -> "DnskeyAlgorithm":
        """Construct item with label."""
        obj = int.__new__(cls, value)
        obj._value_ = value

        obj.label = label
        return obj

    DELETE = (0, _("Delete DS"))
    RSAMD5 = (1, _("RSA/MD5 (deprecated, see 5)"))
    DH = (2, _("Diffie-Helman"))
    DSA = (3, _("DSA/SHA1"))
    _RESERVED_4 = (4, _("Reserved"))
    RSASHA1 = (5, _("RSA/SHA-1"))
    DSA_NSEC3_SHA1 = (6, _("DSA-NSEC3-SHA1"))
    RSASHA1_NSEC3_SHA1 = (7, _("RSASHA1-NSEC3-SHA1"))
    RSASHA256 = (8, _("RSA/SHA-256"))
    _RESERVED_9 = (9, _("Reserved"))
    RSASHA512 = (10, _("RSA/SHA-512"))
    _RESERVED_11 = (11, _("Reserved"))
    ECC_GOST = (12, _("GOST R 34.10-2001"))
    ECDSAP256SHA256 = (13, _("ECDSA Curve P-256 with SHA-256"))
    ECDSAP384SHA384 = (14, _("ECDSA Curve P-384 with SHA-384"))
    ED25519 = (15, _("Ed25519"))
    ED448 = (16, _("Ed448"))
    SM2SM3 = (17, _("SM2 signing algorithm with SM3 hashing algorithm"))
    _UNASSIGNED_18 = (18, _("Unassigned"))
    _UNASSIGNED_19 = (19, _("Unassigned"))
    _UNASSIGNED_20 = (20, _("Unassigned"))
    _UNASSIGNED_21 = (21, _("Unassigned"))
    _UNASSIGNED_22 = (22, _("Unassigned"))
    ECC_GOST12 = (23, _("GOST R 34.10-2012"))
    _UNASSIGNED_24 = (24, _("Unassigned"))
    _UNASSIGNED_25 = (25, _("Unassigned"))
    _UNASSIGNED_26 = (26, _("Unassigned"))
    _UNASSIGNED_27 = (27, _("Unassigned"))
    _UNASSIGNED_28 = (28, _("Unassigned"))
    _UNASSIGNED_29 = (29, _("Unassigned"))
    _UNASSIGNED_30 = (30, _("Unassigned"))
    _UNASSIGNED_31 = (31, _("Unassigned"))
    _UNASSIGNED_32 = (32, _("Unassigned"))
    _UNASSIGNED_33 = (33, _("Unassigned"))
    _UNASSIGNED_34 = (34, _("Unassigned"))
    _UNASSIGNED_35 = (35, _("Unassigned"))
    _UNASSIGNED_36 = (36, _("Unassigned"))
    _UNASSIGNED_37 = (37, _("Unassigned"))
    _UNASSIGNED_38 = (38, _("Unassigned"))
    _UNASSIGNED_39 = (39, _("Unassigned"))
    _UNASSIGNED_40 = (40, _("Unassigned"))
    _UNASSIGNED_41 = (41, _("Unassigned"))
    _UNASSIGNED_42 = (42, _("Unassigned"))
    _UNASSIGNED_43 = (43, _("Unassigned"))
    _UNASSIGNED_44 = (44, _("Unassigned"))
    _UNASSIGNED_45 = (45, _("Unassigned"))
    _UNASSIGNED_46 = (46, _("Unassigned"))
    _UNASSIGNED_47 = (47, _("Unassigned"))
    _UNASSIGNED_48 = (48, _("Unassigned"))
    _UNASSIGNED_49 = (49, _("Unassigned"))
    _UNASSIGNED_50 = (50, _("Unassigned"))
    _UNASSIGNED_51 = (51, _("Unassigned"))
    _UNASSIGNED_52 = (52, _("Unassigned"))
    _UNASSIGNED_53 = (53, _("Unassigned"))
    _UNASSIGNED_54 = (54, _("Unassigned"))
    _UNASSIGNED_55 = (55, _("Unassigned"))
    _UNASSIGNED_56 = (56, _("Unassigned"))
    _UNASSIGNED_57 = (57, _("Unassigned"))
    _UNASSIGNED_58 = (58, _("Unassigned"))
    _UNASSIGNED_59 = (59, _("Unassigned"))
    _UNASSIGNED_60 = (60, _("Unassigned"))
    _UNASSIGNED_61 = (61, _("Unassigned"))
    _UNASSIGNED_62 = (62, _("Unassigned"))
    _UNASSIGNED_63 = (63, _("Unassigned"))
    _UNASSIGNED_64 = (64, _("Unassigned"))
    _UNASSIGNED_65 = (65, _("Unassigned"))
    _UNASSIGNED_66 = (66, _("Unassigned"))
    _UNASSIGNED_67 = (67, _("Unassigned"))
    _UNASSIGNED_68 = (68, _("Unassigned"))
    _UNASSIGNED_69 = (69, _("Unassigned"))
    _UNASSIGNED_70 = (70, _("Unassigned"))
    _UNASSIGNED_71 = (71, _("Unassigned"))
    _UNASSIGNED_72 = (72, _("Unassigned"))
    _UNASSIGNED_73 = (73, _("Unassigned"))
    _UNASSIGNED_74 = (74, _("Unassigned"))
    _UNASSIGNED_75 = (75, _("Unassigned"))
    _UNASSIGNED_76 = (76, _("Unassigned"))
    _UNASSIGNED_77 = (77, _("Unassigned"))
    _UNASSIGNED_78 = (78, _("Unassigned"))
    _UNASSIGNED_79 = (79, _("Unassigned"))
    _UNASSIGNED_80 = (80, _("Unassigned"))
    _UNASSIGNED_81 = (81, _("Unassigned"))
    _UNASSIGNED_82 = (82, _("Unassigned"))
    _UNASSIGNED_83 = (83, _("Unassigned"))
    _UNASSIGNED_84 = (84, _("Unassigned"))
    _UNASSIGNED_85 = (85, _("Unassigned"))
    _UNASSIGNED_86 = (86, _("Unassigned"))
    _UNASSIGNED_87 = (87, _("Unassigned"))
    _UNASSIGNED_88 = (88, _("Unassigned"))
    _UNASSIGNED_89 = (89, _("Unassigned"))
    _UNASSIGNED_90 = (90, _("Unassigned"))
    _UNASSIGNED_91 = (91, _("Unassigned"))
    _UNASSIGNED_92 = (92, _("Unassigned"))
    _UNASSIGNED_93 = (93, _("Unassigned"))
    _UNASSIGNED_94 = (94, _("Unassigned"))
    _UNASSIGNED_95 = (95, _("Unassigned"))
    _UNASSIGNED_96 = (96, _("Unassigned"))
    _UNASSIGNED_97 = (97, _("Unassigned"))
    _UNASSIGNED_98 = (98, _("Unassigned"))
    _UNASSIGNED_99 = (99, _("Unassigned"))
    _UNASSIGNED_100 = (100, _("Unassigned"))
    _UNASSIGNED_101 = (101, _("Unassigned"))
    _UNASSIGNED_102 = (102, _("Unassigned"))
    _UNASSIGNED_103 = (103, _("Unassigned"))
    _UNASSIGNED_104 = (104, _("Unassigned"))
    _UNASSIGNED_105 = (105, _("Unassigned"))
    _UNASSIGNED_106 = (106, _("Unassigned"))
    _UNASSIGNED_107 = (107, _("Unassigned"))
    _UNASSIGNED_108 = (108, _("Unassigned"))
    _UNASSIGNED_109 = (109, _("Unassigned"))
    _UNASSIGNED_110 = (110, _("Unassigned"))
    _UNASSIGNED_111 = (111, _("Unassigned"))
    _UNASSIGNED_112 = (112, _("Unassigned"))
    _UNASSIGNED_113 = (113, _("Unassigned"))
    _UNASSIGNED_114 = (114, _("Unassigned"))
    _UNASSIGNED_115 = (115, _("Unassigned"))
    _UNASSIGNED_116 = (116, _("Unassigned"))
    _UNASSIGNED_117 = (117, _("Unassigned"))
    _UNASSIGNED_118 = (118, _("Unassigned"))
    _UNASSIGNED_119 = (119, _("Unassigned"))
    _UNASSIGNED_120 = (120, _("Unassigned"))
    _UNASSIGNED_121 = (121, _("Unassigned"))
    _UNASSIGNED_122 = (122, _("Unassigned"))
    _RESERVED_123 = (123, _("Reserved"))
    _RESERVED_124 = (124, _("Reserved"))
    _RESERVED_125 = (125, _("Reserved"))
    _RESERVED_126 = (126, _("Reserved"))
    _RESERVED_127 = (127, _("Reserved"))
    _RESERVED_128 = (128, _("Reserved"))
    _RESERVED_129 = (129, _("Reserved"))
    _RESERVED_130 = (130, _("Reserved"))
    _RESERVED_131 = (131, _("Reserved"))
    _RESERVED_132 = (132, _("Reserved"))
    _RESERVED_133 = (133, _("Reserved"))
    _RESERVED_134 = (134, _("Reserved"))
    _RESERVED_135 = (135, _("Reserved"))
    _RESERVED_136 = (136, _("Reserved"))
    _RESERVED_137 = (137, _("Reserved"))
    _RESERVED_138 = (138, _("Reserved"))
    _RESERVED_139 = (139, _("Reserved"))
    _RESERVED_140 = (140, _("Reserved"))
    _RESERVED_141 = (141, _("Reserved"))
    _RESERVED_142 = (142, _("Reserved"))
    _RESERVED_143 = (143, _("Reserved"))
    _RESERVED_144 = (144, _("Reserved"))
    _RESERVED_145 = (145, _("Reserved"))
    _RESERVED_146 = (146, _("Reserved"))
    _RESERVED_147 = (147, _("Reserved"))
    _RESERVED_148 = (148, _("Reserved"))
    _RESERVED_149 = (149, _("Reserved"))
    _RESERVED_150 = (150, _("Reserved"))
    _RESERVED_151 = (151, _("Reserved"))
    _RESERVED_152 = (152, _("Reserved"))
    _RESERVED_153 = (153, _("Reserved"))
    _RESERVED_154 = (154, _("Reserved"))
    _RESERVED_155 = (155, _("Reserved"))
    _RESERVED_156 = (156, _("Reserved"))
    _RESERVED_157 = (157, _("Reserved"))
    _RESERVED_158 = (158, _("Reserved"))
    _RESERVED_159 = (159, _("Reserved"))
    _RESERVED_160 = (160, _("Reserved"))
    _RESERVED_161 = (161, _("Reserved"))
    _RESERVED_162 = (162, _("Reserved"))
    _RESERVED_163 = (163, _("Reserved"))
    _RESERVED_164 = (164, _("Reserved"))
    _RESERVED_165 = (165, _("Reserved"))
    _RESERVED_166 = (166, _("Reserved"))
    _RESERVED_167 = (167, _("Reserved"))
    _RESERVED_168 = (168, _("Reserved"))
    _RESERVED_169 = (169, _("Reserved"))
    _RESERVED_170 = (170, _("Reserved"))
    _RESERVED_171 = (171, _("Reserved"))
    _RESERVED_172 = (172, _("Reserved"))
    _RESERVED_173 = (173, _("Reserved"))
    _RESERVED_174 = (174, _("Reserved"))
    _RESERVED_175 = (175, _("Reserved"))
    _RESERVED_176 = (176, _("Reserved"))
    _RESERVED_177 = (177, _("Reserved"))
    _RESERVED_178 = (178, _("Reserved"))
    _RESERVED_179 = (179, _("Reserved"))
    _RESERVED_180 = (180, _("Reserved"))
    _RESERVED_181 = (181, _("Reserved"))
    _RESERVED_182 = (182, _("Reserved"))
    _RESERVED_183 = (183, _("Reserved"))
    _RESERVED_184 = (184, _("Reserved"))
    _RESERVED_185 = (185, _("Reserved"))
    _RESERVED_186 = (186, _("Reserved"))
    _RESERVED_187 = (187, _("Reserved"))
    _RESERVED_188 = (188, _("Reserved"))
    _RESERVED_189 = (189, _("Reserved"))
    _RESERVED_190 = (190, _("Reserved"))
    _RESERVED_191 = (191, _("Reserved"))
    _RESERVED_192 = (192, _("Reserved"))
    _RESERVED_193 = (193, _("Reserved"))
    _RESERVED_194 = (194, _("Reserved"))
    _RESERVED_195 = (195, _("Reserved"))
    _RESERVED_196 = (196, _("Reserved"))
    _RESERVED_197 = (197, _("Reserved"))
    _RESERVED_198 = (198, _("Reserved"))
    _RESERVED_199 = (199, _("Reserved"))
    _RESERVED_200 = (200, _("Reserved"))
    _RESERVED_201 = (201, _("Reserved"))
    _RESERVED_202 = (202, _("Reserved"))
    _RESERVED_203 = (203, _("Reserved"))
    _RESERVED_204 = (204, _("Reserved"))
    _RESERVED_205 = (205, _("Reserved"))
    _RESERVED_206 = (206, _("Reserved"))
    _RESERVED_207 = (207, _("Reserved"))
    _RESERVED_208 = (208, _("Reserved"))
    _RESERVED_209 = (209, _("Reserved"))
    _RESERVED_210 = (210, _("Reserved"))
    _RESERVED_211 = (211, _("Reserved"))
    _RESERVED_212 = (212, _("Reserved"))
    _RESERVED_213 = (213, _("Reserved"))
    _RESERVED_214 = (214, _("Reserved"))
    _RESERVED_215 = (215, _("Reserved"))
    _RESERVED_216 = (216, _("Reserved"))
    _RESERVED_217 = (217, _("Reserved"))
    _RESERVED_218 = (218, _("Reserved"))
    _RESERVED_219 = (219, _("Reserved"))
    _RESERVED_220 = (220, _("Reserved"))
    _RESERVED_221 = (221, _("Reserved"))
    _RESERVED_222 = (222, _("Reserved"))
    _RESERVED_223 = (223, _("Reserved"))
    _RESERVED_224 = (224, _("Reserved"))
    _RESERVED_225 = (225, _("Reserved"))
    _RESERVED_226 = (226, _("Reserved"))
    _RESERVED_227 = (227, _("Reserved"))
    _RESERVED_228 = (228, _("Reserved"))
    _RESERVED_229 = (229, _("Reserved"))
    _RESERVED_230 = (230, _("Reserved"))
    _RESERVED_231 = (231, _("Reserved"))
    _RESERVED_232 = (232, _("Reserved"))
    _RESERVED_233 = (233, _("Reserved"))
    _RESERVED_234 = (234, _("Reserved"))
    _RESERVED_235 = (235, _("Reserved"))
    _RESERVED_236 = (236, _("Reserved"))
    _RESERVED_237 = (237, _("Reserved"))
    _RESERVED_238 = (238, _("Reserved"))
    _RESERVED_239 = (239, _("Reserved"))
    _RESERVED_240 = (240, _("Reserved"))
    _RESERVED_241 = (241, _("Reserved"))
    _RESERVED_242 = (242, _("Reserved"))
    _RESERVED_243 = (243, _("Reserved"))
    _RESERVED_244 = (244, _("Reserved"))
    _RESERVED_245 = (245, _("Reserved"))
    _RESERVED_246 = (246, _("Reserved"))
    _RESERVED_247 = (247, _("Reserved"))
    _RESERVED_248 = (248, _("Reserved"))
    _RESERVED_249 = (249, _("Reserved"))
    _RESERVED_250 = (250, _("Reserved"))
    _RESERVED_251 = (251, _("Reserved"))
    INDIRECT = (252, _("Reserved for Indirect Keys"))
    PRIVATEDNS = (253, _("Private algorithm"))
    PRIVATEOID = (254, _("Private algorithm OID"))
    _RESERVED_255 = (255, _("Reserved"))
