"""Registry types."""

from enum import unique

from .common import BaseId, BaseModel
from .translations import gettext_lazy as _

try:
    from enum import StrEnum  # type: ignore[attr-defined, unused-ignore]
except ImportError:  # pragma: no cover
    from backports.strenum import StrEnum  # type: ignore[assignment, no-redef, unused-ignore]


class BaseObjectId(BaseId):
    """Registry object identifier base class."""


class BaseObjectHistoryId(BaseId):
    """Registry object history identifier base class."""


class ContactId(BaseObjectId):
    """Contact id."""


class ContactHistoryId(BaseObjectHistoryId):
    """Contact history id."""


class ContactRef(BaseModel):
    """Contact reference info."""

    id: ContactId
    handle: str
    name: str = ""
    organization: str = ""


class DomainId(BaseObjectId):
    """Domain id."""


class DomainHistoryId(BaseObjectHistoryId):
    """Domain history id."""


class DomainRef(BaseModel):
    """Domain reference info."""

    id: DomainId
    fqdn: str


class KeysetId(BaseObjectId):
    """Keyset id."""


class KeysetHistoryId(BaseObjectHistoryId):
    """Keyset history id."""


class KeysetRef(BaseModel):
    """Keyset reference info."""

    id: KeysetId
    handle: str


class NssetId(BaseObjectId):
    """Nsset id."""


class NssetHistoryId(BaseObjectHistoryId):
    """Nsset history id."""


class NssetRef(BaseModel):
    """Nsset reference info."""

    id: NssetId
    handle: str


class RegistrarId(BaseId):
    """Registrar identifier."""


class RepresentativeId(BaseObjectId):
    """Contact representative id."""


class RepresentativeHistoryId(BaseObjectHistoryId):
    """Contact representative history id."""


@unique
class RegistryObjectType(StrEnum):
    """Registry object type."""

    CONTACT = "contact"
    DOMAIN = "domain"
    KEYSET = "keyset"
    NSSET = "nsset"


@unique
class StateFlag(StrEnum):
    """Enum of object state flags."""

    label: str

    def __new__(cls, value: str, label: str = "") -> "StateFlag":
        """Construct item with label."""
        obj = str.__new__(cls, value)
        obj._value_ = value

        obj.label = label
        return obj

    CONDITIONALLY_IDENTIFIED_CONTACT = (
        "conditionallyIdentifiedContact",
        _("Contact is conditionally identified"),
    )
    DELETE_CANDIDATE = ("deleteCandidate", _("To be deleted"))
    DELETE_WARNING = ("deleteWarning", _("The domain will be deleted in 11 days"))
    EXPIRATION_WARNING = ("expirationWarning", _("The domain expires in 30 days"))
    EXPIRED = ("expired", _("Domain expired"))
    IDENTIFIED_CONTACT = ("identifiedContact", _("Contact is identified"))
    LINKED = ("linked", _("Has relation to other records in the registry"))
    NOT_VALIDATED = ("notValidated", _("Domain not validated"))
    NSSET_MISSING = ("nssetMissing", _("The domain doesn't have associated nsset"))
    OUTZONE = ("outzone", _("The domain isn't generated in the zone"))
    OUTZONE_UNGUARDED = (
        "outzoneUnguarded",
        _("The domain is out of zone after 30 days in expiration state"),
    )
    OUTZONE_UNGUARDED_WARNING = (
        "outzoneUnguardedWarning",
        _("The domain is to be out of zone soon."),
    )
    PREMIUM_DOMAIN = ("premiumDomain", _("VIP domain"))
    SERVER_BLOCKED = ("serverBlocked", _("Administratively blocked"))
    SERVER_CONTACT_IDENT_CHANGE_PROHIBITED = (
        "serverContactIdentChangeProhibited",
        _("Ident update forbidden"),
    )
    SERVER_CONTACT_NAME_CHANGE_PROHIBITED = (
        "serverContactNameChangeProhibited",
        _("Name update forbidden"),
    )
    SERVER_CONTACT_ORGANIZATION_CHANGE_PROHIBITED = (
        "serverContactOrganizationChangeProhibited",
        _("Organization update forbidden"),
    )
    SERVER_CONTACT_PERMANENT_ADDRESS_CHANGE_PROHIBITED = (
        "serverContactPermanentAddressChangeProhibited",
        _("Permanent address update forbidden"),
    )
    SERVER_DELETE_PROHIBITED = ("serverDeleteProhibited", _("Deletion forbidden"))
    SERVER_INZONE_MANUAL = ("serverInzoneManual", _("The domain is administratively kept in zone"))
    SERVER_LINK_PROHIBITED = ("serverLinkProhibited", _("Contact linking forbidden"))
    SERVER_OUTZONE_MANUAL = (
        "serverOutzoneManual",
        _("The domain is administratively kept out of zone"),
    )
    SERVER_REGISTRANT_CHANGE_PROHIBITED = (
        "serverRegistrantChangeProhibited",
        _("Registrant change forbidden"),
    )
    SERVER_RENEW_PROHIBITED = ("serverRenewProhibited", _("Registration renewal forbidden"))
    SERVER_TRANSFER_PROHIBITED = (
        "serverTransferProhibited",
        _("Sponsoring registrar change forbidden"),
    )
    SERVER_UPDATE_PROHIBITED = ("serverUpdateProhibited", _("Update forbidden"))
    UNGUARDED = ("unguarded", _("The domain is 30 days after expiration"))
    VALIDATED_CONTACT = ("validatedContact", _("Contact is validated"))
    VALIDATION_WARNING1 = ("validationWarning1", _("The domain validation expires in 30 days"))
    VALIDATION_WARNING2 = ("validationWarning2", _("The domain validation expires in 15 days"))
