"""Translation utilities.

Gettext from standard library doesn't provide support for dynamic language switch,
so it's handled in here.

By default, language can be changed by setting the active language code to `LANGUAGE` context
variable.

Example:
    from fred_types.translations import LANGUAGE

    LANGUAGE.set('cs')

If an integration to another library is required, a custom language getter can be set to
`LANGUAGE_GETTER` context variable.
The language getter is expected to take no arguments and return an active language code.
In such case `LANGUAGE` context variable is ignored.

Example:
    from django.utils.translation import get_language
    from fred_types.translations import LANGUAGE

    LANGUAGE_GETTER.set(get_language)
"""

import gettext
from contextvars import ContextVar
from pathlib import Path
from typing import Any, Callable, Dict, cast

_LOCALE_DIR = Path(__file__).parent / "locale"

_TRANSLATIONS: Dict[str, gettext.NullTranslations] = {}

LANGUAGE: ContextVar[str] = ContextVar("LANGUAGE", default="en")


def _get_language() -> str:
    return LANGUAGE.get()


LANGUAGE_GETTER: ContextVar[Callable[[], str]] = ContextVar(
    "LANGUAGE_GETTER", default=_get_language
)


def _get_translation() -> gettext.NullTranslations:
    current_language = LANGUAGE_GETTER.get()()
    if current_language not in _TRANSLATIONS:
        _TRANSLATIONS[current_language] = gettext.translation(
            "fred_types", localedir=_LOCALE_DIR, fallback=True, languages=[current_language]
        )
    return _TRANSLATIONS[current_language]


class _LazyTranslation:
    def __init__(self, value: str):
        self.value = value

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}: {self.value!r}>"

    def __str__(self) -> str:
        return _get_translation().gettext(self.value)

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, _LazyTranslation):
            other = str(other)
        return str(self) == other  # type: ignore[no-any-return]

    def __ne__(self, other: Any) -> bool:
        return not (self == other)


# Pretend the returned value is string, even though it's actually a _LazyTranslation object.
def gettext_lazy(value: str) -> str:
    """Mark string for lazy translation."""
    return cast(str, _LazyTranslation(value))
